==> lhcsmapi-1.2.19.tar.gz: lhcsmapi-1.2.19/lhcsmapi/dbsignal/cals/CalsDbSignal.py <==
import pandas as pd
from datetime import datetime
from lhcsmapi.Time import Time
from lhcsmapi.dbsignal.DbSignal import DbSignal
import warnings

class CalsDbSignal(DbSignal):
    """CalsDbSignal class encapsulates parsing of a signal obtained from the CALS database with pytimber package.
    """

    def __init__(self, signal={}, t_start=0, t_end=0, ldb=None):
        super().__init__('CALS')
        self.signal_name = signal
        self.t_start = t_start
        self.t_end = t_end
        self._ldb = ldb

        if self._ldb is None:
            raise Exception('You have to provide an instance of pytimber.LoggingDB via the ldb keyword')

    def __str__(self):
        return "Db: CALS, Signal: {}, Start time = {}, End time = {}".format(self.signal_name[0], Time.to_string(self.t_start, 's'), Time.to_string(self.t_end, 's'))

    @staticmethod
    def get_scaled_signal(SIGNAL_NAMES, t_start, t_end, ldb):
        signal_dict = ldb.getScaled(SIGNAL_NAMES, t_start, t_end, scaleAlgorithm='AVG', scaleInterval='MINUTE')
        signal_df = pd.DataFrame.from_dict(signal_dict)
        return signal_df.loc[1, :]

    @staticmethod
    def build_signal(**kwargs):
        """**Builds a PmSignal object**

            Method validates kwargs and returns a CalsSignal object.
            The method looks though kwargs for arguments to create a new CalsSignal. If arguments are not provided, default
            arguments are used

            :param kwargs: dictionary of (keyword: argument) pairs which parametrize an db API query
            :type kwargs: dictionary
            :return: initialized CalsDbSignal object

            - Example::

                from source.post_mortem.PmSignal import PmSignal
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                source = "B20L5"
                signal = "I_HDS_4"
                eventTime = 1426220469491000000
                PmSignal().createSignal(system=system, className=className, source=source,
                              signal=signal, eventTime=eventTime)
               >>> <source.test_cals.CalsSignal.CalsSignal at 0x7f0256e74710>

            - Expected Fail Response::

                from source.test_cals.CalsSignal import CalsSignal
                arcCurrentName = "RPTE.UA47.RB.A45:I_MEAS"
                t_start = 1426220459491000000
                t_end =   1426220479491000000
                CalsSignal.buildSignal(signal=arcCurrentName, t_start = t_start, t_end = t_end)
                >>> Exception: You have to provide a instance of pytimber.LoggingDB via the ldb keyword
        """

        # look through kwargs for constructor arguments
        signal_name = CalsDbSignal.validate_key_in_kwargs('signal', str, tuple, **kwargs) or {}

        t_start = CalsDbSignal.validate_key_in_kwargs('t_start', int, pd.Timestamp, str, float, datetime, tuple, **kwargs) or 0
        t_end = CalsDbSignal.validate_key_in_kwargs('t_end', int, pd.Timestamp, str, float, datetime, tuple, **kwargs)
        duration = CalsDbSignal.validate_key_in_kwargs('duration', tuple, list, **kwargs)
        tz = CalsDbSignal.validate_key_in_kwargs('tz', str, **kwargs)

        # calculate start and end time based of parsed arguments
        t_start, t_end = Time.get_query_period_in_unix_time(t_start, t_end, duration, tz)

        # convert start and end to unix time in seconds with us precision for CALS api
        t_start = Time.to_unix_timestamp_in_sec(t_start, tz=tz)
        t_end = Time.to_unix_timestamp_in_sec(t_end, tz=tz)

        ldb_ = None # different syntax since the pytimber.LoggingDB can't be validated without being a dependency
        if 'ldb' in kwargs:
            ldb_ = kwargs['ldb']

        if isinstance(signal_name, str):
            signal_name = [signal_name]
        return CalsDbSignal(signal_name, t_start, t_end, ldb_)

    def read(self, signal=None, **kwargs):
        """Method returns a pandas DataFrame containing all dbsignal  with an index formatted to ns
        """
        return CalsDbSignal.get_signal_df(self.signal_name, self.t_start, self.t_end, self._ldb)

    @staticmethod
    def get_signal_df(SIGNAL_NAMES, t_start, t_end, ldb):
        """Method returns a pandas DataFrame with dbsignal"""
        signal_df = CalsDbSignal.get_signal(SIGNAL_NAMES[0], t_start, t_end, ldb)
        for s in range(1, len(SIGNAL_NAMES)):
            new_frame = CalsDbSignal.get_signal(SIGNAL_NAMES[s], t_start, t_end, ldb)
            signal_df = pd.concat([signal_df, new_frame], axis=1)
        return signal_df

    @staticmethod
    def get_signal(signal, t_start, t_end, ldb):
        """**Returns a pandas DataFrame with one signalt**

            Method validates kwargs and returns a CalsSignal object.
            The method looks though kwargs for arguments to create a new CalsSignal.
            If arguments are not provided, default arguments are used.

            :param signal: signal signal to be queried
            :type signal: string
            :param t_start: unix seconds representing the start time
            :param t_start: float
            :param t_end: unix seconds representing the end time
            :param t_end: float
            :return: pandas.DataFrame with a single signal

            - Example::

                from source.test_cals.CalsSignal import CalsSignal
                import pytimber
                ldb = pytimber.LoggingDB()

                I_MEAS = "RPTE.UA47.RB.A45:I_MEAS"
                t_start = 1426220459.4910002
                t_end = 1426220479.4910002
                i_meas_df = CalsSignal.getSignal(signal=I_MEAS, t_start=t_start, t_end=t_end, ldb=ldb)
                print(i_meas_df.head())
                >>>               RPTE.UA47.RB.A45:I_MEAS
                >>> 1.426220e+09                  10071.5
                >>> 1.426220e+09                  10076.5
                >>> 1.426220e+09                  10081.5
                >>> 1.426220e+09                  10086.5
                >>> 1.426220e+09                  10091.5

            - Expected Fail Response::

                from source.test_cals.CalsSignal import CalsSignal
                import pytimber
                ldb = pytimber.LoggingDB()
                arcCurrentName = "RPTE.UA47.RB.A45:I_MEA"
                t_start = 1426220459.4910002
                t_end =   1426220479.4910002
                I_MEAS = CalsSignal.getSignal(signal=arcCurrentName, t_start = t_start,
                                       t_end = t_end, ldb = ldb)
                print(I_MEAS.head())
                >>> Empty DataFrame
                >>> Columns: []
                >>> Index: []
                >>> UserWarning: Querying CALS with the parameters signal: RPTE.UA47.RB.A45:I_MEA, start time: 1426220459.4910002 and end time: 1426220479.4910002 returned no data.
                >>> An empty DataFrame was returned.
                >>> ___________________________________________________________________________________
                >>> Reproduce the error using the raw pytimber API:
                >>>
                >>> import pytimber
                >>> ldb = pytimber.LoggingDB()
                >>> ldb.get("RPTE.UA47.RB.A45:I_MEA", 1426220459.4910002, 1426220479.4910002)
        """
        try:
            d = ldb.get(signal, t_start, t_end)
        except:
            exception = """Querying CALS with the parameters signal: {0}, start time: {1} and end time: {2} failed
            ___________________________________________________________________________________
            Reproduce the error using the raw pytimber API:\n
            import pytimber
            ldb = pytimber.LoggingDB()
            ldb.get(\"{0}\", {1}, {2})""".format(signal, t_start, t_end)

            raise Exception(exception)
        
        if d.get(signal) is None:
            warning = """Querying CALS with the parameters signal: {0}, start time: {1} and end time: {2} returned no data.
            An empty DataFrame is returned.
            ___________________________________________________________________________________
            Reproduce the error using the raw pytimber API:\n
            import pytimber
            ldb = pytimber.LoggingDB()
            ldb.get(\"{0}\", {1}, {2})""".format(signal, t_start, t_end)

            warnings.warn(warning)
        
            return pd.DataFrame()

        return pd.DataFrame(d.get(signal)[1], d.get(signal)[0], [signal])

==> lhcsmapi-1.2.111.tar.gz: lhcsmapi-1.2.111/lhcsmapi/dbsignal/cals/CalsDbSignal.py <==
import pandas as pd
from datetime import datetime
from lhcsmapi.Time import Time
from lhcsmapi.dbsignal.DbSignal import DbSignal
import warnings

class CalsDbSignal(DbSignal):
    """CalsDbSignal class encapsulates parsing of a signal obtained from the CALS database with pytimber package.
    """

    def __init__(self, signal={}, t_start=0, t_end=0, ldb=None):
        super().__init__('CALS')
        self.signal_name = signal
        self.t_start = t_start
        self.t_end = t_end
        self._ldb = ldb

        if self._ldb is None:
            raise Exception('You have to provide an instance of pytimber.LoggingDB via the ldb keyword')

    def __str__(self):
        return "Db: CALS, Signal: {}, Start time = {}, End time = {}".format(self.signal_name[0], Time.to_string(self.t_start, 's'), Time.to_string(self.t_end, 's'))

    @staticmethod
    def get_scaled_signal(SIGNAL_NAMES, t_start, t_end, ldb):
        signal_dict = ldb.getScaled(SIGNAL_NAMES, t_start, t_end, scaleAlgorithm='AVG', scaleInterval='MINUTE')
        signal_df = pd.DataFrame.from_dict(signal_dict)
        return signal_df.loc[1, :]

    @staticmethod
    def build_signal(**kwargs):
        """**Builds a PmSignal object**

            Method validates kwargs and returns a CalsSignal object.
            The method looks though kwargs for arguments to create a new CalsSignal. If arguments are not provided, default
            arguments are used

            :param kwargs: dictionary of (keyword: argument) pairs which parametrize an db API query
            :type kwargs: dictionary
            :return: initialized CalsDbSignal object

            - Example::

                from source.post_mortem.PmSignal import PmSignal
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                source = "B20L5"
                signal = "I_HDS_4"
                eventTime = 1426220469491000000
                PmSignal().createSignal(system=system, className=className, source=source,
                              signal=signal, eventTime=eventTime)
               >>> <source.test_cals.CalsSignal.CalsSignal at 0x7f0256e74710>

            - Expected Fail Response::

                from source.test_cals.CalsSignal import CalsSignal
                arcCurrentName = "RPTE.UA47.RB.A45:I_MEAS"
                t_start = 1426220459491000000
                t_end =   1426220479491000000
                CalsSignal.buildSignal(signal=arcCurrentName, t_start = t_start, t_end = t_end)
                >>> Exception: You have to provide a instance of pytimber.LoggingDB via the spark keyword
        """

        # look through kwargs for constructor arguments
        signal_name = CalsDbSignal.validate_key_in_kwargs('signal', str, tuple, **kwargs) or {}

        t_start = CalsDbSignal.validate_key_in_kwargs('t_start', int, pd.Timestamp, str, float, datetime, tuple, **kwargs) or 0
        t_end = CalsDbSignal.validate_key_in_kwargs('t_end', int, pd.Timestamp, str, float, datetime, tuple, **kwargs)
        duration = CalsDbSignal.validate_key_in_kwargs('duration', tuple, list, **kwargs)
        tz = CalsDbSignal.validate_key_in_kwargs('tz', str, **kwargs)

        # calculate start and end time based of parsed arguments
        t_start, t_end = Time.get_query_period_in_unix_time(t_start, t_end, duration, tz)

        # convert start and end to unix time in seconds with us precision for CALS api
        t_start = Time.to_unix_timestamp_in_sec(t_start, tz=tz)
        t_end = Time.to_unix_timestamp_in_sec(t_end, tz=tz)

        ldb_ = None # different syntax since the pytimber.LoggingDB can't be validated without being a dependency
        if 'ldb' in kwargs:
            ldb_ = kwargs['ldb']

        if isinstance(signal_name, str):
            signal_name = [signal_name]
        return CalsDbSignal(signal_name, t_start, t_end, ldb_)

    def read(self, signal=None, **kwargs):
        """Method returns a pandas DataFrame containing all dbsignal  with an index formatted to ns
        """
        return CalsDbSignal.get_signal_df(self.signal_name, self.t_start, self.t_end, self._ldb)

    @staticmethod
    def get_signal_df(SIGNAL_NAMES, t_start, t_end, ldb):
        """Method returns a pandas DataFrame with dbsignal"""
        signal_df = CalsDbSignal.get_signal(SIGNAL_NAMES[0], t_start, t_end, ldb)
        for s in range(1, len(SIGNAL_NAMES)):
            new_frame = CalsDbSignal.get_signal(SIGNAL_NAMES[s], t_start, t_end, ldb)
            signal_df = pd.concat([signal_df, new_frame], axis=1)
        return signal_df

    @staticmethod
    def get_signal(signal, t_start, t_end, ldb):
        """**Returns a pandas DataFrame with one signalt**

            Method validates kwargs and returns a CalsSignal object.
            The method looks though kwargs for arguments to create a new CalsSignal.
            If arguments are not provided, default arguments are used.

            :param signal: signal signal to be queried
            :type signal: string
            :param t_start: unix seconds representing the start time
            :param t_start: float
            :param t_end: unix seconds representing the end time
            :param t_end: float
            :return: pandas.DataFrame with a single signal

            - Example::

                from source.test_cals.CalsSignal import CalsSignal
                import pytimber
                ldb = pytimber.LoggingDB()

                I_MEAS = "RPTE.UA47.RB.A45:I_MEAS"
                t_start = 1426220459.4910002
                t_end = 1426220479.4910002
                i_meas_df = CalsSignal.get_signal(signal=I_MEAS, t_start=t_start, t_end=t_end, ldb=ldb)
                print(i_meas_df.head())
                >>>               RPTE.UA47.RB.A45:I_MEAS
                >>> 1.426220e+09                  10071.5
                >>> 1.426220e+09                  10076.5
                >>> 1.426220e+09                  10081.5
                >>> 1.426220e+09                  10086.5
                >>> 1.426220e+09                  10091.5

            - Expected Fail Response::

                from source.test_cals.CalsSignal import CalsSignal
                import pytimber
                ldb = pytimber.LoggingDB()
                arcCurrentName = "RPTE.UA47.RB.A45:I_MEA"
                t_start = 1426220459.4910002
                t_end =   1426220479.4910002
                I_MEAS = CalsSignal.getSignal(signal=arcCurrentName, t_start = t_start,
                                       t_end = t_end, ldb = ldb)
                print(I_MEAS.head())
                >>> Empty DataFrame
                >>> Columns: []
                >>> Index: []
                >>> UserWarning: Querying CALS with the parameters signal: RPTE.UA47.RB.A45:I_MEA, start time: 1426220459.4910002 and end time: 1426220479.4910002 returned no data.
                >>> An empty DataFrame was returned.
                >>> ___________________________________________________________________________________
                >>> Reproduce the error using the raw pytimber API:
                >>>
                >>> import pytimber
                >>> ldb = pytimber.LoggingDB()
                >>> ldb.get("RPTE.UA47.RB.A45:I_MEA", 1426220459.4910002, 1426220479.4910002)
        """
        try:
            d = ldb.get(signal, t_start, t_end)
        except:
            exception = """Querying CALS with the parameters signal: {0}, start time: {1} and end time: {2} failed
            ___________________________________________________________________________________
            Reproduce the error using the raw pytimber API:\n
            import pytimber
            ldb = pytimber.LoggingDB()
            ldb.get(\"{0}\", {1}, {2})""".format(signal, t_start, t_end)

            raise Exception(exception)
        
        if d.get(signal) is None:
            warning = """Querying CALS with the parameters signal: {0}, start time: {1} and end time: {2} returned no data.
            An empty DataFrame is returned.
            ___________________________________________________________________________________
            Reproduce the error using the raw pytimber API:\n
            import pytimber
            ldb = pytimber.LoggingDB()
            ldb.get(\"{0}\", {1}, {2})""".format(signal, t_start, t_end)

            warnings.warn(warning)
        
            return pd.DataFrame()

        return pd.DataFrame(d.get(signal)[1], d.get(signal)[0], [signal])
